require 'csv'
require 'json'
require 'rest-client'
require 'retryable'

module Adk2
  class Client
    attr_reader :api_token, :username, :password

    def initialize(network_name, opts = {})
      @api_base_url = "https://#{network_name}.adk2.com"
      @username = opts[:username]
      @password = opts[:password]
      @api_token = opts[:api_token]
    end

    def entities(type, params = {})
      JSON.parse(http_get(type, { rows: 1000 }.merge(params)), symbolize_names: true)[:rows]
    end

    def entity(type, id)
      JSON.parse(http_get("#{type}/#{id}"), symbolize_names: true)
    end

    def report(level, name, start_date, end_date)
      response = http_get("reports/#{level}/#{name}/#{start_date}-#{end_date}", fmt: 'csv')
      CSV.parse(response, headers: true).map(&:to_h)
    end

    def get_api_token
      return @api_token = nil if username.nil? || password.nil?
      response = resource('auth').post(
        {
          'email' => username,
          'password' => password
        }.to_json
      )

      @api_token = resource('profile/apikey').post(
        '',
        cookies: response.cookies
      )
    end

    private

    attr_reader :api_base_url, :api_token

    def http_get(path, params = {})
      get_api_token if api_token.nil?

      Retryable.retryable(
        tries: 5,
        sleep: -> n { 4**n },
        on: [RestClient::Forbidden, RestClient::ServiceUnavailable]
      ) do |_, exception|
        get_api_token if exception.is_a?(RestClient::Forbidden)

        resource("denver/#{path}").get(
          params: { fmt: 'json', auth: api_token }.merge(params)
        )
      end
    end

    def resource(path)
      RestClient::Resource.new(
        "#{api_base_url}/#{path}",
        ssl_ca_file: File.expand_path('../../../adk2.crt', __FILE__)
      )
    end
  end
end
